@@ -50,13 +50,12 @@ class DataServiceMixin:
         "timestamp": "string",
     }
     
-    @classmethod
-    def _get_field_type(cls, column_info):
+    def _get_field_type(self, column_info):
         """Convert database column type to frontend display type"""
         return (
             "string"
             if column_info["name"] in ["id", "sys_id"]
-            else cls._pg_types.get(column_info["type_name"], "string")
+            else self._pg_types.get(column_info["type_name"], "string")
         )
     
     # ========== Hook Methods (Override These) ==========
