@@ -1,8 +1,8 @@
 import sys
+import time
 import traceback
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional
 from velocity.misc.format import to_json
-from support.app import DEBUG
 
 
 class Response:
@@ -13,7 +13,8 @@ class Response:
     def __init__(self):
         """Initialize the Response object with default status, headers, and an empty actions list."""
         self.actions: List[Dict[str, Any]] = []
-        self.body: Dict[str, Any] = {"actions": self.actions}
+        self.jobs = JobEnvelope(self)
+        self.body: Dict[str, Any] = {}
         self.raw: Dict[str, Any] = {
             "statusCode": 200,
             "body": "{}",
@@ -32,6 +33,10 @@ class Response:
         Returns:
             Dict[str, Any]: The complete HTTP response with headers, status code, and JSON body.
         """
+        if self.actions:
+            self.body["actions"] = self.actions
+        if self.jobs.events:
+            self.body["jobs"] = {"events": self.jobs.events}
         self.raw["body"] = to_json(self.body)
         return self.raw
 
@@ -202,8 +207,8 @@ class Response:
                 "python_exception": {
                     "type": str(exc_type),
                     "value": str(exc_value),
-                    "traceback": traceback.format_exc() if DEBUG else None,
-                    "tb": traceback.format_tb(tb) if DEBUG else None,
+                    "traceback": traceback.format_exc(),
+                    "tb": traceback.format_tb(tb) if tb else [],
                 }
             }
         )
@@ -288,3 +293,184 @@ class Response:
             str: The formatted header key.
         """
         return "-".join(word.capitalize() for word in key.split("-"))
+
+
+class JobEnvelope:
+    """Structured helper for emitting job lifecycle events in API responses."""
+
+    def __init__(self, response: "Response") -> None:
+        self._response = response
+        self.events: List[Dict[str, Any]] = []
+
+    def start(
+        self,
+        *,
+        job_id: Any,
+        label: Optional[str] = None,
+        job_key: Optional[str] = None,
+        status: Optional[str] = None,
+        detail: Optional[str] = None,
+        auto_download: Optional[bool] = True,
+        poll_interval: Optional[int] = 3000,
+        batch_id: Optional[str] = None,
+        metadata: Optional[Dict[str, Any]] = None,
+        cancellable: Optional[bool] = None,
+        cancel_action: Optional[str] = None,
+    ) -> Dict[str, Any]:
+        """Record the starting state for a long-running job."""
+
+        job = self._build_job(
+            job_id,
+            label=label,
+            job_key=job_key,
+            status=status or "Queued",
+            detail=detail or "Queued",
+            auto_download=auto_download,
+            poll_interval=poll_interval,
+            batch_id=batch_id,
+            metadata=metadata,
+            cancellable=cancellable,
+            cancel_action=cancel_action,
+        )
+        return self._record_event("start", job)
+
+    def progress(
+        self,
+        *,
+        job_id: Any,
+        percent: Optional[float] = None,
+        detail: Optional[str] = None,
+        status: Optional[str] = None,
+        stage: Optional[str] = None,
+        metadata: Optional[Dict[str, Any]] = None,
+    ) -> Dict[str, Any]:
+        """Report progress for an in-flight job."""
+
+        job = self._build_job(
+            job_id,
+            percent=percent,
+            detail=detail,
+            status=status,
+            stage=stage,
+            metadata=metadata,
+            partial=True,
+        )
+        return self._record_event("progress", job)
+
+    def complete(
+        self,
+        *,
+        job_id: Any,
+        detail: Optional[str] = None,
+        result: Optional[Any] = None,
+        metadata: Optional[Dict[str, Any]] = None,
+        auto_download: Optional[bool] = None,
+    ) -> Dict[str, Any]:
+        """Mark the job as completed."""
+
+        job = self._build_job(
+            job_id,
+            status="Completed",
+            detail=detail,
+            result=result,
+            metadata=metadata,
+            auto_download=auto_download,
+            partial=True,
+        )
+        return self._record_event("complete", job)
+
+    def fail(
+        self,
+        *,
+        job_id: Any,
+        detail: Optional[str] = None,
+        error: Optional[str] = None,
+        metadata: Optional[Dict[str, Any]] = None,
+    ) -> Dict[str, Any]:
+        """Mark the job as failed."""
+
+        job = self._build_job(
+            job_id,
+            status="Failed",
+            detail=detail or error,
+            error=error,
+            metadata=metadata,
+            partial=True,
+        )
+        return self._record_event("failed", job)
+
+    def cancel(
+        self,
+        *,
+        job_id: Any,
+        detail: Optional[str] = None,
+        metadata: Optional[Dict[str, Any]] = None,
+    ) -> Dict[str, Any]:
+        """Mark the job as cancelled."""
+
+        job = self._build_job(
+            job_id,
+            status="Cancelled",
+            detail=detail,
+            metadata=metadata,
+            partial=True,
+        )
+        return self._record_event("cancelled", job)
+
+    def _record_event(self, event_type: str, job: Dict[str, Any]) -> Dict[str, Any]:
+        if not job.get("jobId"):
+            raise ValueError("jobId is required for job events")
+        self.events.append(
+            {
+                "type": event_type,
+                "job": job,
+                "timestamp": int(time.time() * 1000),
+            }
+        )
+        return job
+
+    def _build_job(
+        self,
+        job_id: Any,
+        *,
+        label: Optional[str] = None,
+        job_key: Optional[str] = None,
+        status: Optional[str] = None,
+        detail: Optional[str] = None,
+        auto_download: Optional[bool] = None,
+        poll_interval: Optional[int] = None,
+        batch_id: Optional[str] = None,
+        percent: Optional[float] = None,
+        stage: Optional[str] = None,
+        result: Optional[Any] = None,
+        metadata: Optional[Dict[str, Any]] = None,
+        cancellable: Optional[bool] = None,
+        cancel_action: Optional[str] = None,
+        error: Optional[str] = None,
+        partial: bool = False,
+    ) -> Dict[str, Any]:
+        job: Dict[str, Any] = {"jobId": str(job_id)}
+
+        def assign(key: str, value: Any) -> None:
+            if value is not None:
+                job[key] = value
+
+        assign("jobKey", job_key)
+        assign("label", label)
+        assign("status", status)
+        assign("detail", detail)
+        assign("autoDownload", auto_download)
+        assign("pollInterval", poll_interval)
+        assign("batchId", batch_id)
+        assign("percent", percent)
+        assign("stage", stage)
+        assign("result", result)
+        assign("metadata", metadata)
+        assign("cancellable", cancellable)
+        assign("cancelAction", cancel_action)
+        assign("error", error)
+
+        if partial and "detail" not in job and status:
+            job["detail"] = status
+
+        return job
